# -*- coding: utf-8 -*-
"""
@author: Mathias Henriksen
Doi: 
"""
import os
import datetime
import cantera as ct
import auxiliaryFunctions as af
import pandas as pd
"""
-------------------------------------------------------------------------------
Function related to writing the thermoProperties support documentation.
"""

def write_support_documentation_file_for_thermoProperties_file_with_3_species(mainPath,inputParameters,oxidant,fuel,burntProducts,report):
    os.chdir(mainPath) 
    now = datetime.datetime.now(); date=now.strftime("%Y_%m_%d")
    saveName = 'thermo_'+inputParameters['name of combustion mixture']+'_'+inputParameters['reactionMech'][:-4]+'_'+date+'.txt'
    file = open(saveName,'w+')
    intro = write_intro_in_support_documentation_file_for_thermoProperties_file(date); file.writelines(intro)
    skipOneLine = [' \n']; file.writelines(skipOneLine)
    now = datetime.datetime.now(); date = now.strftime("%m/%d/%Y, %H:%M:%S")
    inputSection=write_common_input_section_in_support_documentation_file_for_thermoProperties_file_for_3_species(date,inputParameters); file.writelines(inputSection)
    file.writelines(skipOneLine)
    phiAndMassFractionSection=write_phi_mass_fraction_list_in_support_documentation_file_for_thermoProperties_file_for_3_species(inputParameters); file.writelines(phiAndMassFractionSection)
    file.writelines(skipOneLine)
    oxidantSection = write_specie_input_section_in_support_documentation_file_for_thermoProperties('oxidant',oxidant,report['oxidant']); file.writelines(oxidantSection)
    file.writelines(skipOneLine)
    fuelSection = write_specie_input_section_in_support_documentation_file_for_thermoProperties('fuel',fuel,report['fuel']); file.writelines(fuelSection)
    file.writelines(skipOneLine) 
    burntProductsSection = write_specie_input_section_in_support_documentation_file_for_thermoProperties('burntProducts',burntProducts,report['burntProducts']); file.writelines(burntProductsSection)
    file.writelines(skipOneLine)
    file.close()
    "write csv file"
    df = write_nasa_polynomial_csv_file(inputParameters,oxidant,fuel,burntProducts)
    df.to_csv(saveName[:-4]+'.csv',index=False)
    return saveName

def write_intro_in_support_documentation_file_for_thermoProperties_file(date):
    string = ['This is a support document for thermophysicalProperties generated on '+date+' \n',
             'It includes the parameters used to generate the NASA polynomials and \n',
             'calculate the Sutherland Coefficients. It alsocontains the NASA polynomials \n',
             'and Sutherland Coefficients for verification of thermophysicalProperties file. \n',
             'To sort between different thermophysicalProperties files, store this support \n',
             'document alongside the thermophysicalProperties file generated. \n',
             ]
    return string

def write_common_input_section_in_support_documentation_file_for_thermoProperties_file_for_3_species(date,inputs):
    string = ['*\   This section contains the parameter used to generate thermophysicalProperties file   /* \n',
              ' \n',
              'Date created                                 '+date+' \n',
              'Reaction Mechanism                           '+inputs['reactionMech']+' \n',
              'Fuel(s)                                      '+inputs['fuel']+' \n',
              'Oxidant                                      '+inputs['oxidant']+' \n',
              'Burnt Products                               '+inputs['burntProducts']+' \n',
              'Stocihmetric fuel mass fraction              '+str(inputs['Stoich Fuel mass fraction'])+' \n',
              'Stocihmetric Air/Fuel mass ratio             '+str(inputs['Stoich AirFuel mass ratio'])+' \n',
              'Reference Tempurature                        '+str(inputs['T0'])+' \n',
              'Reference pressure                           '+str(inputs['p0'])+' \n',
              ]
    return string

def write_phi_mass_fraction_list_in_support_documentation_file_for_thermoProperties_file_for_3_species(inputs):
    oldPath = os.getcwd()
    os.chdir(inputs['reaction mech path'])
    gas = ct.Solution(inputs['reactionMech'])
    os.chdir(oldPath)
    phiAndMassFracionList = af.get_fuel_mass_fraction_for_a_range_of_equivalence_ratios(gas,inputs['fuel'],inputs['oxidant'],
                                                                                        inputs['T0'],inputs['p0'])
    string = ['*\   This section contains the parameter used to generate thermophysicalProperties file   /* \n',
              '\n',]
    phi, fraction = phiAndMassFracionList[0].split(':')
    string+=['Equivalence ratio : Fuel Mass Fraction       '+phi+' : '+fraction+' \n',]
    for phiAndMassFrac in phiAndMassFracionList[1:]:
        phi, fraction = phiAndMassFrac.split(':')
        string+=['                                             '+phi+' : '+fraction+' \n',]
    return string

def write_specie_input_section_in_support_documentation_file_for_thermoProperties(specie,inputs,report):
    string = ['*\   This section contains the  thermophysical properties generate for the '+specie+' /* \n'
              ' \n',
              'Reactants average mole weight                '+str(inputs['mole weight'])+' \n',
              'Minimum Temperature for NASA poly            '+str(inputs['minimum ref temp'])+' \n',
              'Comments for minimum temperature check       '+report['minimum ref temp']+' \n',
              'Common Temperature NASA poly                 '+str(inputs['common ref temp'])+' \n',
              'Comments for common temperature check        '+report['common ref temp']+' \n',                  
              'Maximum Temperature NASA poly                '+str(inputs['maximum ref temp'])+' \n',
              'Comments for maximum temperature check       '+report['maximum ref temp']+' \n', 
              'Sutherland Coefficient As                    '+str(inputs['As'])+' \n',
              'Sutherland Coefficient Ts                    '+str(inputs['Ts'])+' \n',
              'High coefficients for NASA Polynomials       '+str(inputs['NASApoly high Coefficients'][0])+' \n',
              '                                             '+str(inputs['NASApoly high Coefficients'][1])+' \n',
              '                                             '+str(inputs['NASApoly high Coefficients'][2])+' \n',
              '                                             '+str(inputs['NASApoly high Coefficients'][3])+' \n',
              '                                             '+str(inputs['NASApoly high Coefficients'][4])+' \n',
              '                                             '+str(inputs['NASApoly high Coefficients'][5])+' \n',
              '                                             '+str(inputs['NASApoly high Coefficients'][6])+' \n',
              'Low coefficients for NASA Polynomials        '+str(inputs['NASApoly low Coefficients'][0])+' \n',
              '                                             '+str(inputs['NASApoly low Coefficients'][1])+' \n',
              '                                             '+str(inputs['NASApoly low Coefficients'][2])+' \n',
              '                                             '+str(inputs['NASApoly low Coefficients'][3])+' \n',
              '                                             '+str(inputs['NASApoly low Coefficients'][4])+' \n',
              '                                             '+str(inputs['NASApoly low Coefficients'][5])+' \n',
              '                                             '+str(inputs['NASApoly low Coefficients'][6])+' \n',
              ]
    return string

def write_nasa_polynomial_dataframe_for_specie(specie,inputs):
    lowCoefficients = list(inputs['NASApoly low Coefficients'])
    highCoefficients = list(inputs['NASApoly high Coefficients'])
    dataframe = pd.DataFrame({specie+' - NASApoly low Coefficients': lowCoefficients,
                              specie+' - NASApoly high Coefficients' : highCoefficients})
    return dataframe

def write_nasa_polynomial_csv_file(inputParameters,oxidant,fuel,burntProducts):
    df  = pd.DataFrame({'name of combustion mixture': [inputParameters['name of combustion mixture']]})
    df1 = write_nasa_polynomial_dataframe_for_specie('oxidant',oxidant)
    df2 = write_nasa_polynomial_dataframe_for_specie('fuel',fuel)
    df3 = write_nasa_polynomial_dataframe_for_specie('burntProducts',burntProducts) 
    dataframe = pd.concat([df,df1,df2,df3], ignore_index=True, axis=1)
    dataframe.columns = ['name of combustion mixture','oxidant - NASApoly high Coefficients',
                         'oxidant - NASApoly low Coefficients','fuel - NASApoly high Coefficients',
                         'fuel - NASApoly low Coefficients','burntProducts - NASApoly high Coefficients',
                         'burntProducts - NASApoly low Coefficients']
    
    return dataframe




"""
End of function related to writing the thermoProperties support documentation.
-------------------------------------------------------------------------------
Function related to writing the Gulder output document
"""

def write_intro_for_gulder_documentation_file(date):
    string = ['This document contains the Gulder coefficients, the input data used and was\n',
              'created the '+date+'. The section containing the Gulder coefficients can be \n',
              'copied directly into combustionProperties file, under the last Gulder \n',
              'coefficients. This file is ment to be stored together with combustionProperties \n',
              'file as documentation \n',
             ]
    return string

def write_input_section_for_gulder_documentation_file(date,inputs):
    string = ['*\      This section contains the input used to generate the Gulder Coefficient     /* \n'
              ' \n',
              'Date created                                 '+date+' \n',
              'Name of combustion mixture                   '+inputs['name of combustion mixture']+' \n',
              'Reaction Mechanism                           '+inputs['reaction mech']+' \n',
              'Fuel(s)                                      '+inputs['fuel']+' \n',
              'Oxidant                                      '+inputs['oxidant']+' \n',                        
              'Fuel-Air Equivalnce ratio range              '+inputs['equivalence ratio range for fit']+' \n',
              'Reference Laminar burning velocity           '+str(inputs['reference LBV at phi 1.0'])+' \n',
              'Temperature range                            '+inputs['equivalence ratio range for fit']+' \n',
              'Reference Tempurature                        '+str(inputs['temperatue range for fit'])+' \n',  
              'Pressure range                               '+inputs['pressure range for fit']+' \n',
              'Reference Pressure                           '+str(inputs['reference pressure'])+' \n',
              ]
    return string

def write_gulder_coefficients_table_for_gulder_documentation_file(gulder,inputs):
    string = ['   '+inputs['name of combustion mixture']+'\n',
            '   { \n',
            '       W               '+str(gulder['omega'])+'; \n',
            '       eta             '+str(gulder['eta'])+'; \n',
            '       xi              '+str(gulder['xi'])+'; \n',
            '       alpha           '+str(gulder['alpha'])+'; \n',
            '       beta            '+str(gulder['beta'])+'; \n',
            '       f               '+'2.3; // Constant, see Gulders article for documentation \n',            
            '   } \n'
            ]
    return string

def create_dataframe_for_LBV_csv(inputParameters,LBV):
    df  = pd.DataFrame({'name of combustion mixture': [inputParameters['name of combustion mixture']]})
    df1 = pd.DataFrame({'LBV - at Constant T,p': LBV['phi'],
                        'LBV - range of phi' : LBV['phi list']})
    df2 = pd.DataFrame({'LBV - at Constant phi,p': LBV['temp'],
                        'LBV - range of temp' : LBV['temp list']})
    df3 = pd.DataFrame({'LBV - at Constant T,phi': LBV['pressure'],
                        'LBV - range of press' : LBV['pressure list']})
    dataframe = pd.concat([df,df1,df2,df3], ignore_index=True, axis=1)
    dataframe.columns = ['Mixture Name','LBV at constant T,p','LBV - range of phi',
                         'LBV at constant phi,p','LBV - range of temperature',
                         'LBV constant T,phi','LBV - range of pressure']
    return dataframe

def write_gulder_output_document_and_LBV_csv(path,inputParameters,gulderCoefficients,LBV):
    os.chdir(path)
    now = datetime.datetime.now(); date=now.strftime("%Y_%m_%d")
    saveName = 'Gulders_'+inputParameters['name of combustion mixture']+'_'+inputParameters['reaction mech'][:-4]+'_'+date+'.txt'
    gulderFile = open(saveName,'w+')
    introGulder = write_intro_for_gulder_documentation_file(date); gulderFile.writelines(introGulder)
    skipOneLines = [' \n']; gulderFile.writelines(skipOneLines) 
    now = datetime.datetime.now(); date=now.strftime("%m/%d/%Y, %H:%M:%S")
    inputsGulder = write_input_section_for_gulder_documentation_file(date,inputParameters); gulderFile.writelines(inputsGulder) 
    gulderFile.writelines(skipOneLines) 
    coefGulder = write_gulder_coefficients_table_for_gulder_documentation_file(gulderCoefficients,inputParameters); gulderFile.writelines(coefGulder) 
    gulderFile.writelines(skipOneLines)
    gulderFile.close() 
    "write laminar burning velocities used for fitting the gulder coefficients"
    LBVdataframe = create_dataframe_for_LBV_csv(inputParameters,LBV)
    currentPath=os.getcwd()
    LBVdataframe.to_csv(currentPath+'\\'+saveName[:-4]+'.csv',index=False)
    return 
