# -*- coding: utf-8 -*-
"""
Code in article
@author: Mathias Heniksen 
doi: article
"""
import os
import writeSupportDocumentations as wsd
#For inhomogenousMixture

def write_thermoProperties_file_for_Xioam_for_inhomogenous_combustion(mainPath,inputParameters,oxidant,fuel,burntProducts,report,writeSupportFile='yes'):
    os.chdir(mainPath)
    if writeSupportFile == 'yes':
        supportFilename = wsd.write_support_documentation_file_for_thermoProperties_file_with_3_species(mainPath,inputParameters,oxidant,fuel,burntProducts,report)
    FoamName = 'thermophysicalProperties'   
    tfile = open(FoamName,'w+')
    intro = write_intro_for_thermoProperties(); tfile.writelines(intro)
    skipOneLine = [' \n']; tfile.writelines(skipOneLine)
    thermoType= write_inhomogeneous_mixture_thermo_parameters(); tfile.writelines(thermoType)
    tfile.writelines(skipOneLine)
    stoichString = write_stoichiometric_AirFuel_mass_ratio(inputParameters); tfile.writelines(stoichString) 
    tfile.writelines(skipOneLine)
    oxidantString = write_specie_entry_in_thermoProperties('oxidant',oxidant); tfile.writelines(oxidantString)
    fuelString = write_specie_entry_in_thermoProperties('fuel',fuel); tfile.writelines(fuelString)
    burntProductsString = write_specie_entry_in_thermoProperties('burntProducts',burntProducts); tfile.writelines(burntProductsString)
    tfile.writelines(skipOneLine)
    if writeSupportFile == 'yes':
        traceFile = ['// -For information about the values in this document, see file see '+supportFilename+'-''\n']; tfile.writelines(traceFile)
        tfile.writelines(skipOneLine)
    endFoam = ['// ************************************************************************* // \n']; tfile.writelines(endFoam)
    tfile.close()
    return

def write_specie_entry_in_thermoProperties(specieType,inputs):
    if inputs['minimum ref temp'] > 200:
        print('Changed the minimum reference temperature for '+specieType+', from '+str(inputs['minimum ref temp'])+'K to 200 K. \n',
              'This is done to avoid temperature warnings when running OpenFOAM')
        inputs['minimum ref temp'] = 200

    thermFoam = [''+specieType+' \n',
                 '{ \n',
                 '    specie \n',
                 '    { \n',
                 '        molWeight       '+str(inputs['mole weight'])+'; \n',
                 '    } \n',
                 '    thermodynamics \n',
                 '    { \n',
                 '        Tlow            '+str(inputs['minimum ref temp'])+'; \n',
                 '        Thigh           '+str(inputs['maximum ref temp'])+'; \n',
                 '        Tcommon         '+str(inputs['common ref temp'])+'; \n',
                 '        highCpCoeffs \n',
                 '        ( \n',
                 '            '+str(inputs['NASApoly high Coefficients'][0])+' '+str(inputs['NASApoly high Coefficients'][1])+' '+str(inputs['NASApoly high Coefficients'][2])+' '+str(inputs['NASApoly high Coefficients'][3])+'\n',
                 '            '+str(inputs['NASApoly high Coefficients'][4])+' '+str(inputs['NASApoly high Coefficients'][5])+' '+str(inputs['NASApoly high Coefficients'][6])+'\n',
                 '        ); \n',
                 '        lowCpCoeffs \n',
                 '        ( \n',
                 '            '+str(inputs['NASApoly low Coefficients'][0])+' '+str(inputs['NASApoly low Coefficients'][1])+' '+str(inputs['NASApoly low Coefficients'][2])+' '+str(inputs['NASApoly low Coefficients'][3])+'\n',
                 '            '+str(inputs['NASApoly low Coefficients'][4])+' '+str(inputs['NASApoly low Coefficients'][5])+' '+str(inputs['NASApoly low Coefficients'][6])+'\n',
                 '        ); \n',
                 '    } \n',
                 '    transport \n',
                 '    { \n',
                 '        As              '+str(inputs['As'])+'; \n',
                 '        Ts              '+str(inputs['Ts'])+'; \n',
                 '    } \n',
                 '} \n',
                 ]
    return thermFoam

def write_stoichiometric_AirFuel_mass_ratio(inputParameters):
    string = ['stoichiometricAirFuelMassRatio \n',
              '    stoichiometricAirFuelMassRatio [0 0 0 0 0 0 0] '+str(inputParameters['Stoich AirFuel mass ratio'])+'; \n']
    return string

def write_intro_for_thermoProperties():
    intro     = ['/*--------------------------------*- C++ -*----------------------------------*\ \n', 
                 '  =========                 | \n', 
                 '  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox \n', 
                 '   \\    /   O peration     | Website:  https://openfoam.org \n',
                 '    \\  /    A nd           | Version:  6 \n',
                 '     \\/     M anipulation  | \n',
                 '\*---------------------------------------------------------------------------*/ \n'
                 'FoamFile \n'
                 '{ \n'
                 '    version     2.0; \n'
                 '    format      ascii; \n'
                 '    class       dictionary; \n'
                 '    location    "constant"; \n'
                 '    object      thermophysicalProperties; \n'
                 '} \n'
                 '// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * // \n'
                 ]
    return intro

def write_inhomogeneous_mixture_thermo_parameters():
    thermoType = ['thermoType \n',
                  '{ \n',
                  '    type            heheuPsiThermo; \n',
                  '    mixture         inhomogeneousMixture; \n',
                  '    transport       sutherland; \n',
                  '    thermo          janaf; \n',
                  '    equationOfState perfectGas; \n',
                  '    specie          specie; \n',
                  '    energy          absoluteEnthalpy; \n',
                  '} \n'
                  ]
    return thermoType